#version 400

layout(points, invocations=1) in;
//layout(triangle_strip, max_vertices=30) out;
//layout(points, max_vertices=1) out;
layout(triangle_strip,max_vertices=6) out;

uniform mat4 mvpMatrix;
uniform vec3 antPos1;
uniform vec3 antPos2;
uniform float sichtweite;

in vec4 vPosition[1];
in vec4 vColor[1];
in uint visVisible[1];

out vec4 gColor;
out vec2 gUV;

/* Cubic Particle Offset */
//const float offset = 0.2f;
//const vec4 v1off = vec4(-offset, offset, offset, 0.0f);
//const vec4 v2off = vec4( offset, offset, offset, 0.0f);
//const vec4 v3off = vec4( offset,-offset, offset, 0.0f);
//const vec4 v4off = vec4(-offset,-offset, offset, 0.0f);
//const vec4 v5off = vec4( offset, offset,-offset, 0.0f);
//const vec4 v6off = vec4( offset,-offset,-offset, 0.0f);
//const vec4 v7off = vec4(-offset, offset,-offset, 0.0f);
//const vec4 v8off = vec4(-offset,-offset,-offset, 0.0f);

/* UV coords */
const vec2 lo = vec2(0.0f,0.0f);
const vec2 ro = vec2(1.0f,0.0f);
const vec2 ru = vec2(1.0f,1.0f);
const vec2 lu = vec2(0.0f,1.0f);

/* Billboard Particle Offset */
const float offset = 0.35f;
const vec4 v1 = vec4(-offset, offset, 0.0f,0.0f);
const vec4 v2 = vec4(offset, offset, 0.0f,0.0f);
const vec4 v3 = vec4(offset, -offset, 0.0f,0.0f);
const vec4 v4 = vec4(-offset, -offset, 0.0f,0.0f);

void main()
{	
	if(visVisible[0]==0) return;

	vec4 point = gl_in[0].gl_Position;

	//float antDistance = min(distance(antPos1.xz,point.xz),distance(antPos2.xz,point.xz));

	//if(antDistance < sichtweite) return;
	
	/* Point Particle
	gl_Position = mvpMatrix * point;
	gColor = vColor[0];
	gUV = vec2(0.5f,0.5f);

	EmitVertex();
	*/

	/* Billboard Particle */
	point = mvpMatrix * point;

	gl_Position = point + v1;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = point + v2;
	gColor = vColor[0];
	gUV = ro;

	EmitVertex();
	

	gl_Position = point + v3;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();
	EndPrimitive();

	gl_Position = point + v1;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = point + v3;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	gl_Position = point + v4;
	gColor = vColor[0];
	gUV = lu;

	EmitVertex();

	/* Cubic Particle
	vec4 v1 = mvpMatrix * (point + v1off);
	vec4 v2 = mvpMatrix * (point + v2off);
	vec4 v3 = mvpMatrix * (point + v3off);
	vec4 v4 = mvpMatrix * (point + v4off);
	vec4 v5 = mvpMatrix * (point + v5off);
	vec4 v6 = mvpMatrix * (point + v6off);
	vec4 v7 = mvpMatrix * (point + v7off);
	vec4 v8 = mvpMatrix * (point + v8off);

	//Cube Front
	gl_Position = v1;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v2;
	gColor = vColor[0];
	gUV = ro;

	EmitVertex();

	gl_Position = v3;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	EndPrimitive();

	gl_Position = v1;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v3;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	gl_Position = v4;
	gColor = vColor[0];
	gUV = lu;
	
	EmitVertex();

	EndPrimitive();
	
	//Cube Right
	gl_Position = v2;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v5;
	gColor = vColor[0];
	gUV = ro;

	EmitVertex();

	gl_Position = v6;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	EndPrimitive();

	gl_Position = v2;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v6;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	gl_Position = v3;
	gColor = vColor[0];
	gUV = lu;
	
	EmitVertex();

	EndPrimitive();

	//Cube Back
	gl_Position = v5;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v7;
	gColor = vColor[0];
	gUV = ro;

	EmitVertex();

	gl_Position = v8;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	EndPrimitive();

	gl_Position = v5;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v8;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	gl_Position = v6;
	gColor = vColor[0];
	gUV = lu;
	
	EmitVertex();

	EndPrimitive();

	//Cube Left
	gl_Position = v7;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v1;
	gColor = vColor[0];
	gUV = ro;

	EmitVertex();

	gl_Position = v4;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	EndPrimitive();

	gl_Position = v7;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v4;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	gl_Position = v8;
	gColor = vColor[0];
	gUV = lu;
	
	EmitVertex();

	EndPrimitive();

	//Cube Top
	gl_Position = v7;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v5;
	gColor = vColor[0];
	gUV = ro;

	EmitVertex();

	gl_Position = v2;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	EndPrimitive();

	gl_Position = v7;
	gColor = vColor[0];
	gUV = lo;

	EmitVertex();

	gl_Position = v2;
	gColor = vColor[0];
	gUV = ru;

	EmitVertex();

	gl_Position = v1;
	gColor = vColor[0];
	gUV = lu;
	
	EmitVertex();

	EndPrimitive();
	*/
}